#!/usr/bin/python

##   Previews Reader
##   Copyright (C) 2004 Mads Villadsen <maxx@krakoa.dk>
    
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
   
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
   
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gobject, libxml2, os, re, string

from globals import *
import util

try:
    import pygtk
    pygtk.require ("2.0")
    import gtk, gtk.glade, gnome, gnome.ui
except:
    print "pyGTK 2.0 not found."

COLUMN_ORDER = 0
COLUMN_TITLE = 1
COLUMN_ID_HIDDEN = 2

class pr_mainwindow:
    def __init__(self):
        self.wTree = None
        try:
            self.wTree = gtk.glade.XML (gladefile, "main_window")
        except:
            self.wTree = gtk.glade.XML (devel_gladefile, "main_window")
        self.main_window = self.wTree.get_widget ("main_window")
        self.previews_item_list = []
        self.show_star_system_items = False
        self.group_by_publisher = False
        
        dic = {
            "on_main_window_destroy" : gtk.mainquit,
            "on_quit_activate" : gtk.mainquit,
            "on_print_order_list1_activate" : self.print_order_list,
            "on_view_order_list_activate" : self.view_order_list,
            "on_group_by_publisher_activate" : self.group_by_publisher_activate,
            "on_show_star_system_items_activate" : self.show_star_system_items_activate,
            "on_about_activate" : self.about_activate,
            "on_open_activate" : self.open_activate,
            "on_open_treeview_row_activated" : self.open_treeview_row_activated,
            "on_treeview_row_activated" : self.show_description
            }
        self.wTree.signal_autoconnect (dic)
        
        # Favorites DOM
        try:
            self.fav_doc = libxml2.parseFile (fav_fname)
            self.fav_root = self.fav_doc.xpathEval("/favorites")[0]
            if self.fav_doc.name != fav_fname:
                print "doc.name error"
                sys.exit(1)
        except:
            self.fav_doc = libxml2.newDoc("1.0")
            self.fav_root = self.fav_doc.newChild(None, "favorites", None)

        # setup treeview
        self.previewsTreeView = self.wTree.get_widget ("treeview1")
        self.previewsTreeModel = gtk.TreeStore (gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.previewsTreeView.set_model (self.previewsTreeModel)
        self.previewsTreeView.set_headers_visible (True)
        self.previewsTreeView.set_search_column(COLUMN_TITLE)

        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.order_toggled, self.previewsTreeModel)
        column = gtk.TreeViewColumn('Order', renderer, active=COLUMN_ORDER)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(60)
        self.previewsTreeView.append_column (column)
        
        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Title", renderer, text=COLUMN_TITLE)
        column.set_resizable (True)
        self.previewsTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("previews item id (hidden)", renderer, text=COLUMN_ID_HIDDEN)
        column.set_resizable (True)

        self.previewsTreeView.show ()
        return

    def insert_row (self, model, parent, title, id):
        iter = model.insert_before (parent, None)

        if (self.default_orders and self.item_is_favorite(id)):
            item = self.previews_item_list[int(id)]
            val = item.get_title()
            val = self.clean(val)
            # does it already exist in the order DOM?
            order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
            if (len(order_node) == 0):
                # add to order DOM
                elem = self.order_root.newChild(None, "order", None)
                elem.setProp("title", val)
                elem.setProp("itemcode", item.get_itemcode())

        order = self.item_is_order(id)

        model.set_value (iter, COLUMN_ORDER, order)
        model.set_value (iter, COLUMN_TITLE, util.pretty_title(title))
        model.set_value (iter, COLUMN_ID_HIDDEN, id)
        return iter

    def fill_rows (self, previews_items):
        self.previewsTreeModel.clear()

        parent_iters = {}

        star_system_match_re = re.compile("\([A-Z][A-Z][A-Z][A-Z]?[0-9]?[0-9][0-9][0-9][0-9][0-9]\)")
        for i,v in enumerate(previews_items):
            if (self.show_star_system_items or star_system_match_re.search(v.get_title()) == None):
                if (not parent_iters.has_key(v.get_publisher())):
                    tmp_iter = None
                    if (self.group_by_publisher):
                        tmp_iter = self.previewsTreeModel.insert_before (None, None)
                        self.previewsTreeModel.set_value (tmp_iter, COLUMN_ORDER, False)
                        self.previewsTreeModel.set_value (tmp_iter, COLUMN_TITLE, util.pretty_title(v.get_publisher()))
                        self.previewsTreeModel.set_value (tmp_iter, COLUMN_ID_HIDDEN, -1)
                    
                    parent_iters[v.get_publisher()] = tmp_iter

                iter = self.insert_row (self.previewsTreeModel,
                                        parent_iters[v.get_publisher()],
                                        v.get_title(),
                                        i)
        return
	
    def clean(self, raw):
        raw = string.replace(raw, '&', '')
        raw = string.replace(raw, '<', '')
        raw = string.replace(raw, '>', '')
        raw = string.replace(raw, "'", '')
        raw = string.replace(raw, '"', '')
        return raw

    def favorite_toggled(self, widget, id):
        favorite = self.item_is_favorite(id)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            if (string.find(val, "#") >= 0):
                val = val[0:string.find(val, "#")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            if (favorite == False):
                # set current item as a favorite
                favorite = True

                # does it already exist in favorites?
                fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
                if (len(fav_node) == 0):
                    # add to favorites dom
                    elem = self.fav_root.newChild(None, "favorite", None)
                    elem.setProp("title", val)
            else:
                favorite = False
                # remove current item from favorites dom
                fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
                # only remove if it hasn't already been removed
                if (len(fav_node) != 0):
                    fav_node[0].unlinkNode()

            # set new value
            widget.set_active(favorite)
            # save the dom to a file
            self.fav_doc.saveFile(fav_fname)

    def item_is_favorite(self, id):
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            if (string.find(val, "#") >= 0):
                val = val[0:string.find(val, "#")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            # does it already exist in favorites?
            fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
            if (len(fav_node) == 1):
                return True
            else:
                return False

    def item_is_order(self, id):
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            # does it already exist in orders DOM?
            order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
            if (len(order_node) == 1):
                return True
            else:
                return False

    def order_toggled(self, cell, path, model):
        iter = model.get_iter(path)
        order = model.get_value(iter, COLUMN_ORDER)

        id =  self.previewsTreeModel.get_value (iter, COLUMN_ID_HIDDEN)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (order == False):
                # set status of current item as order
                order = True

                # does it already exist in the order DOM?
                order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
                if (len(order_node) == 0):
                    # add to order DOM
                    elem = self.order_root.newChild(None, "order", None)
                    elem.setProp("title", val)
                    elem.setProp("itemcode", item.get_itemcode())
            else:
                order = False
                # remove current item from order DOM
                order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
                # only remove if it hasn't already been removed
                if (len(order_node) != 0):
                    order_node[0].unlinkNode()

            # set new value
            model.set(iter, COLUMN_ORDER, order)
            # save the dom to a file
            self.order_doc.saveFile(self.orders_fname)
            self.default_orders = False

    def show_description (self, widget, path, column):
        iter = self.previewsTreeModel.get_iter (path)
        id =  self.previewsTreeModel.get_value (iter, COLUMN_ID_HIDDEN)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        
        descriptionTextView = self.wTree.get_widget ("textview1")
        buffer = descriptionTextView.get_buffer ()
        buffer.delete (buffer.get_start_iter (), buffer.get_end_iter ())
        iter = buffer.get_start_iter ()

        if (int(id) >= 0):
            buffer.insert(iter, "\n")
            anchor = buffer.create_child_anchor(iter)
            buffer.insert(iter, "\n\n")
            favorites_button = gtk.CheckButton()
            favorites_button.set_label("Favorite")
            favorites_button.set_active(self.item_is_favorite(id))
            favorites_button.connect("toggled", self.favorite_toggled, id)
            descriptionTextView.add_child_at_anchor(favorites_button, anchor)
            favorites_button.show_all()

            buffer.insert (iter, "Title: " + util.pretty_title(item.get_title()) + "\n")
            buffer.insert (iter, "Publisher: " + util.pretty_title(item.get_publisher()) + "\n")
            buffer.insert (iter, "Price: " + item.get_price() + "\n")
            if (item.get_comment() != ""):
                buffer.insert (iter, "Comment: " + item.get_comment() + "\n")
            buffer.insert (iter, "Located on page " + item.get_pagenumber() + " in the Previews catalog.")

            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            search_item_re = re.compile("[\t]*" + val, re.IGNORECASE)
            for e in self.previews_string_list:
                e = unicode(e, "cp1252")
                clean = e
                clean = string.replace(clean, ":", "")
                clean = string.replace(clean, ".", "")
                clean = string.replace(clean, ",", "")
                clean = string.replace(clean, "!", "")
                clean = string.replace(clean, "'S", "")
                clean = string.replace(clean, "'", "")

                search_item_match = search_item_re.match(clean)
                if (search_item_match != None):
                    buffer.insert (iter, "\n\nDescription:\n\n")
                    buffer.insert (iter, e)
                    break

        return

    def print_order_list(self, widget):
        print "Not supported."
        return

    def view_order_list(self, widget):
        view_order_dialogwTree = None
        try:
            view_order_dialogwTree = gtk.glade.XML(gladefile, "view_order_dialog")
        except:
            view_order_dialogwTree = gtk.glade.XML(devel_gladefile, "view_order_dialog")
        view_order_dialog = view_order_dialogwTree.get_widget("view_order_dialog")
        
        orderTextView = view_order_dialogwTree.get_widget ("order_textview")
        buffer = orderTextView.get_buffer ()
        buffer.delete (buffer.get_start_iter (), buffer.get_end_iter ())
        iter = buffer.get_start_iter ()

        for i,v in enumerate(self.previews_item_list):
            if (self.item_is_order(i)):
                buffer.insert(iter, v.pretty_string())
                buffer.insert(iter, "\n")

        result = view_order_dialog.run()
        view_order_dialog.destroy()

    def about_activate(self, widget):
        about_dialogwTree = None
        try:
            about_dialogwTree = gtk.glade.XML(gladefile, "about")
        except:
            about_dialogwTree = gtk.glade.XML(devel_gladefile, "about")
        about_dialog = about_dialogwTree.get_widget("about")
        about_dialog.set_property('name', program_name)
        about_dialog.set_property('version', program_version)

    def open_activate(self, widget):
        open_dialogwTree = None
        try:
            open_dialogwTree = gtk.glade.XML(gladefile, "open_dialog")
        except:
            open_dialogwTree = gtk.glade.XML(devel_gladefile, "open_dialog")
        open_dialog = open_dialogwTree.get_widget("open_dialog")
        
        dic = {
            "on_open_treeview_row_activated" : self.open_treeview_row_activated
            }
        open_dialogwTree.signal_autoconnect (dic)

        openTreeView = open_dialogwTree.get_widget ("open_treeview")
        openListModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        openTreeView.set_model(openListModel)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Previews Issue", renderer, text=0)
        openTreeView.append_column (column)

        previews = os.listdir (previews_dir)
        if (len(previews) > 0):
            previews.sort()
            for fname in previews:
                if (fname[-13:] == "orderform.txt"):
                    date = fname[9:16]
                    iter = openListModel.append (None)
                    openListModel.set_value (iter, 0, "Previews " + " " + date)
                    openListModel.set_value (iter, 1, date)
        else:
            date = "0000-00"
            iter = openListModel.append (None)
            openListModel.set_value (iter, 0, "No Previews files found")
            openListModel.set_value (iter, 1, date)

        result = open_dialog.run()

        if (result == gtk.RESPONSE_OK):
            selected = openTreeView.get_selection().get_selected()
            iter = selected[1]
            if (iter != None):
                date_to_open = selected[0].get_value(iter, 1)
                if (date_to_open != "0000-00"):
                    # Order DOM
                    try:
                        self.orders_fname = basedir + "order-" + date_to_open + ".xml"
                        self.order_doc = libxml2.parseFile (self.orders_fname)
                        self.order_root = self.order_doc.xpathEval("/orders")[0]
                        self.default_orders = False
                        if self.order_doc.name != basedir + "order-" + date_to_open + ".xml":
                            print "doc.name error"
                            sys.exit(1)
                    except:
                        self.order_doc = libxml2.newDoc("1.0")
                        self.order_root = self.order_doc.newChild(None, "orders", None)
                        self.default_orders = True

                    self.previews_string_list = util.get_previews_data (previews_dir + "previews-" + date_to_open)
                    self.previews_item_list = util.get_orderform_items (previews_dir + "previews-" + date_to_open + "-orderform.txt")
                    self.fill_rows (self.previews_item_list)
                    self.main_window.set_title(program_name + " - " + date_to_open)

        open_dialog.destroy()

    def open_treeview_row_activated(self, treeview, path, column):
        selected = treeview.get_selection().get_selected()
        iter = selected[1]
        if (iter != None):
            date_to_open = selected[0].get_value(iter, 1)
            if (date_to_open != "0000-00"):
                # Order DOM
                try:
                    self.orders_fname = basedir + "order-" + date_to_open + ".xml"
                    self.order_doc = libxml2.parseFile (self.orders_fname)
                    self.order_root = self.order_doc.xpathEval("/orders")[0]
                    self.default_orders = False
                    if self.order_doc.name != basedir + "order-" + date_to_open + ".xml":
                        print "doc.name error"
                        sys.exit(1)
                except:
                    self.order_doc = libxml2.newDoc("1.0")
                    self.order_root = self.order_doc.newChild(None, "orders", None)
                    self.default_orders = True

                self.previews_string_list = util.get_previews_data (previews_dir + "previews-" + date_to_open)
                self.previews_item_list = util.get_orderform_items (previews_dir + "previews-" + date_to_open + "-orderform.txt")
                self.fill_rows (self.previews_item_list)
                self.main_window.set_title(program_name + " - " + date_to_open)
            
            treeview.get_parent().get_parent().get_parent().get_parent().response(gtk.RESPONSE_OK)


    def group_by_publisher_activate(self, widget):
        self.group_by_publisher = widget.get_active()
        self.fill_rows (self.previews_item_list)


    def show_star_system_items_activate(self, widget):
        self.show_star_system_items = widget.get_active()
        self.fill_rows (self.previews_item_list)
